unit AstroUtils;

interface
         Function  JulianDay (y, m, d : WORD) : LONGINT;
         Procedure JulianDayToDate (nd : LONGINT; VAR y, m, d : SmallInt);

         Function  MoonAge(y,m,d:word):Extended;
         Function  Sun_Position(y,m,d:word):Extended;
         Function  Moon_Position(y,m,d:word):Extended;
implementation

{Function MoonAge                                      }
{Computes age of Moon (in days)                        }
{INPUT:                                                }
{       y - year                                       }
{       m - month                                      }
{       d - day                                        }
Function MoonAge(y,m,d:word):Extended;
const
     Moon_Orbit_Time  = 29.53058;
     Moon_Orbit_Speed = 4.867;
var
   j:LongInt;
   Age:Extended;
begin
     j:=JulianDay(y,m,d);
     Age:=(J+Moon_Orbit_Speed)/ Moon_Orbit_Time;
     Age:=2*(Age-Int(Age))-1;

     if Age<0 then
        Age:=Moon_Orbit_Time*(1+Age/2)
     else
        Age:=Age*Moon_Orbit_Time/2;

     if Age>=29.53058 then Age:=0;
     result:=Age;
end;

{function Sun Position}
{return the position of the sun on the ecliptic}
Function Sun_Position(y,m,d:word):Extended;
begin
     result:=abs((JulianDay(y,m,d)-JulianDay(y,03,23))*(360/365.25));
end;

Function Moon_Position(y,m,d:word):Extended;
var
   MA:Extended;
begin
     MA:=MoonAge(y,m,d);
     result:=Sun_Position(y,m,d)+MA*(360/29.53058);
end;


{ procedure JulianDAY from CACM Alorithm 199                             }
{ Computes Julian day number for any Gregorian Calendar date             }
{ GIVEN:   y - year                                                      }
{          m - month                                                     }
{          d - day                                                       }
{ RETURNS: Julian day number (astronomically, for the day                }
{          beginning at noon) on the given date.                         }
Function JulianDay (y, m, d : WORD) : LONGINT;

  VAR
    Tmp1, Tmp2, Tmp3, Tmp4, Tmp5 : LONGINT;
  BEGIN
    IF m > 2 THEN
      BEGIN
        Tmp1 := m - 3;
        Tmp2 := y
      END
    ELSE
      BEGIN
        Tmp1 := m + 9;
        Tmp2 := y - 1
      END;
    Tmp3 := Tmp2 DIV 100;
    Tmp4 := Tmp2 MOD 100;
    Tmp5 := d;
    JulianDay := (146097 * Tmp3) DIV 4 + (1461 * Tmp4) DIV 4 +
                 (153 * Tmp1 + 2) DIV 5 + Tmp5 + 1721119
  END;  { JulianDay }


Procedure JulianDayToDate (nd : LONGINT; VAR y, m, d : SmallInt);
  { procedure JDATE from CACM Algorithm 199                                }
  { Computes calendar date from a given Julian day number for any          }
  { valid Gregorian calendar date                                          }
  { GIVEN:   nd - Julian day number (2440000 --> 1968 5 23)                }
  { RETURNS: y - year                                                      }
  {          m - month                                                     }
  {          d - day                                                       }
VAR
    Tmp1, Tmp2, Tmp3 : LONGINT;
  BEGIN
    Tmp1 := nd - 1721119;
    Tmp3 := (4 * Tmp1 - 1) DIV 146097;
    Tmp1 := (4 * Tmp1 - 1) MOD 146097;
    Tmp2 := Tmp1 DIV 4;
    Tmp1 := (4 * Tmp2 + 3) DIV 1461;
    Tmp2 := (4 * Tmp2 + 3) MOD 1461;
    Tmp2 := (Tmp2 + 4) DIV 4;
    m := ((5 * Tmp2 - 3) DIV 153);
    Tmp2 := (5 * Tmp2 - 3) MOD 153;
    d := ((Tmp2 + 5) DIV 5);
    y := (100 * Tmp3 + Tmp1);
    IF m < 10 THEN
      m := m + 3
    ELSE
      BEGIN
        m := m - 9;
        y := y + 1
      END
  END;  { JulianDayToDate }

End.
