unit StarUtils;

interface
uses Forms,Windows,Math,Graphics,SysUtils,AstroUtils;

Const
     Scale:byte = 3;

type
    TSky = Object
    private
           CenterSphereX,CenterSphereY:integer;
           procedure Calc_XY_from_RA_Dec(RA,Dec:Extended;var X,Y:integer);
    public
//----------------------------------------------------------------------------//
//                              Main  utils                                   //
//----------------------------------------------------------------------------//
          constructor Create;
          procedure Draw_Star(Sender:TObject;
                    RA,Dec,Angle:Extended;Magnitude,Option:byte;Name:String);
          procedure Draw_Constellation(Sender:TObject;
                    RAFrom,DecFrom,RATo,DecTo,Angle:Extended);
          procedure Draw_Equator(Sender:TObject);
          procedure Draw_Poles(Sender:TObject);
          procedure Draw_Ecliptic(Sender:TObject);
          procedure Draw_Text_Align(C: TCanvas; A: Single);
          procedure Draw_Constellation_Names(Sender:TObject);
          procedure Draw_Grid(Sender:TObject;StepLong,StepLat:word);

          procedure Draw_Sun(Sender:TObject;BMP:TBitmap);
          procedure Draw_Moon(Sender:TObject;BMP:TBitmap);
//---------------------------------------------------------------------------//
//                              Misc utils                                   //
//---------------------------------------------------------------------------//
          procedure TraceCoordEcliptic(Sender:TObject;BMP:TBitmap;Position,AngleFromEcliptic:Extended);
    end;


implementation

uses Stars_1;

constructor TSky.Create;
var
   F:File of byte;
   j:integer;
   NR:integer;
begin
     ZeroMemory(@Star_Array,SizeOF(Star_Array));
     ZeroMemory(@Cons_Array,SizeOF(Cons_Array));
     ZeroMemory(@Cons_Names,SizeOF(Cons_Names));


     AssignFile(F,'consname.bin');   //Constellation database
     Reset(F);
     j:=0;
     repeat
           BlockRead(F,Cons_Array[j],SizeOF(TConstellation),NR);
           inc(j);
     until SizeOF(TConstellation)>NR;
     CloseFile(F);

     j:=0;

     AssignFile(F,'starmap.tx_');    //YALE StarMap database
     Reset(F);
     repeat
          BlockRead(F,Star_Array[j],SizeOF(TStarRec),NR);
          inc(j);
     until SizeOF(TStarRec)>NR;
     CloseFile(F);

     j:=0;

     AssignFile(F,'cname.bin');      //Database with constellation names
     Reset(F);
     repeat
          BlockRead(F,Cons_Names[j],SizeOF(TConstellationName),NR);
          inc(j);
     until SizeOF(TConstellationName)>NR;
     CloseFile(F);

     CenterSphereX :=Screen.Width div 2;
     CenterSphereY :=Screen.Height div 2;

end;

procedure TSky.Draw_Text_Align(C: TCanvas; A: Single);
var
  LogRec: TLOGFONT;     {Font information}
begin
  GetObject(C.Font.Handle,SizeOf(LogRec),@LogRec);
  LogRec.lfEscapement := Trunc(A*10);
  C.Font.Handle := CreateFontIndirect(LogRec);
end;

//************ Main subroutine (calc X,Y form RA and Dec)*********************//
procedure TSky.Calc_XY_from_RA_Dec(RA,Dec:Extended;var X,Y:integer);
Function roteta2x (ro,teta,rayMax : word) : integer;
begin
     result:=round(cos(DegToRad(teta*0.0054932))*ro*raymax/65535);
end;
Function roteta2y (ro,teta,rayMax : word) : integer;
begin
     result:=round(sin(DegToRad(teta*0.0054932))*ro*raymax/65535);
end;
var
ta,td                    : extended;
tta,ttd                  : word;
azm,htr,x_tmp,y_tmp,z_tmp: extended;
Cos_Lat,Sin_Lat,cos_htr,Sin_htr:Extended;
begin
     ta:=RA*15;
     td:=Dec;
     // ******************************************************//
     azm:=degtorad(360-ta);
     htr:=degtorad(td);
     SinCos(DegToRad(Latitude),Sin_Lat,Cos_Lat); //2x faster
     SinCos(Htr               ,Sin_htr,Cos_Htr); //2x faster

     x_tmp:= Cos_Lat * sin_htr - Sin_Lat * cos(azm) * cos_htr;
     y_tmp:=-sin(azm)* cos_htr;
     z_tmp:= Sin_Lat * sin_htr + Cos_Lat * cos_htr * cos(azm);

     azm:=180*arctan2(y_tmp,x_tmp)/pi;
     if (azm<0) then
       azm:=azm+360;
     htr:=180*arctan(z_tmp/(sqrt(sqr(x_tmp)+sqr(y_tmp))))/pi;
     ta:=azm;
     td:=htr;
     // ******************************************************//
     If td >0 then
       begin
            ttd:=round( (180-td)*65535/90 );
            tta:=round( ta*65535/360 );
            x:=CenterSphereX+roteta2x(ttd,tta+65535 shr 2,CenterSphereY);
            y:=CenterSphereY-roteta2y(ttd,tta+65535 shr 2,CenterSphereY);
       end
     else
       begin
            x:=-100;
            y:=-100;
       end;

end;

procedure TSky.Draw_Constellation_Names(Sender:TObject);
var
   i:integer;
   X,Y:integer;
begin
i:=-1;
(Sender as TBitmap).Canvas.Font.Color:=clYellow;
(Sender as TBitmap).Canvas.Font.Size:=1;

      repeat
           inc(i);
           Calc_XY_From_RA_Dec(cons_names[i].Ra/1000+Ugol_Rot/15,cons_names[i].Dec/100,X,Y);
           (Sender as TBitmap).Canvas.TextOut(X,Y,cons_names[i].Name);
      until cons_names[i+1].Name='';

end;

procedure TSky.Draw_Star(Sender:TObject;
          RA,Dec,Angle:Extended;Magnitude,Option:byte;Name:String);
var
   X,Y:Integer;
   M:byte;
begin
     M:=0;
     case Magnitude of
          0:M:=5;
          1:M:=3;
          2:M:=2;
          3:M:=1;
     end;

     Calc_XY_from_RA_Dec(Ra+Angle/15,Dec,x,y);


     (Sender as TBitmap).Canvas.Ellipse(X-(M div 2),Y-(M div 2),
                                           X+(M div 2),Y+(M div 2));

     if (Option=Magnitude) and ((x>0) and (y>0)) then
     begin
          (Sender as TBitmap).Canvas.Font.Color:=clRed;
          (Sender as TBitmap).Canvas.Font.Size:=3;

          (Sender as TBitmap).Canvas.Brush.Style:=bsClear;
          (Sender as TBitmap).Canvas.TextOut(X+2,Y+2,Name);
     end;
end;

procedure TSky.Draw_Constellation(Sender:TObject;
                             RAFrom,DecFrom,RATo,DecTo,Angle:Extended);
var
   x,y,x1,y1:integer;
begin
     Calc_XY_from_RA_Dec(RaFrom+Angle/15,DecFrom,x,y);

     Calc_XY_from_RA_Dec(RaTo+Angle/15,DecTo,x1,y1);
     if ((x<0) and (y<0)) or ((x1<0) and (y1<0)) then exit;
     (Sender as TBitmap).Canvas.MoveTo(x,y);
     (Sender as TBitmap).Canvas.LineTo(x1,y1);



end;


procedure TSky.Draw_Equator(Sender:TObject);
var
   a:Extended;
   x,y:integer;
begin
     with (Sender as TBitmap).Canvas do
     begin
//Equator
          Pen.Color:=clGreen;
          Pen.Mode:=pmCopy;
          Brush.Style:=bsClear;
          Font.Color:=clLime;
          a:=0; // 
          repeat
                Calc_XY_From_RA_Dec(a/15,0,X,Y);
                (Sender as TBitmap).Canvas.Pixels[x,y]:=clTeal;
                a:=a+0.1;
          until a>360;

//Text
          a:=0; // 
          repeat
                Calc_XY_From_RA_Dec(a+Ugol_Rot/15,0,X,Y);
                if (x>0) and (y>0) then (Sender as TBitmap).Canvas.TextOut(x,y,IntToStr(trunc(a) mod 24) );
                a:=a+1;
          until a>360;

          Font.Color:=clRed;
     end;
end;

procedure TSky.Draw_Poles(Sender:TObject);
var
   x,y:integer;
begin
     with (Sender as TBitmap).Canvas do
     begin
          Pen.Color:=clLime;
          Calc_XY_from_RA_Dec(0,90,x,y);
          Pixels[x,y]:=clLime;

          MoveTo(X,Y+5);
          LineTo(X,Y-5);

          MoveTo(X+5,Y);
          LineTo(X-5,Y);
     end;
end;

procedure TSky.Draw_Ecliptic(Sender:TObject);
var
   x,y:integer;
   i:integer;
   dec:Extended;
begin
     for i:=0 to 2400 do
     begin
          dec:=sin(DegToRad(i/100*15))*23.5;
          Calc_XY_From_RA_Dec(i/100+Ugol_Rot/15,dec,x,y);
          (Sender as TBitmap).Canvas.Pixels[x,y]:=clMaroon;
     end;
end;

procedure TSky.TraceCoordEcliptic(Sender:TObject;BMP:TBitmap;Position,AngleFromEcliptic:Extended);
var
   x,y:integer;
   dec:Extended;
begin
//Position in degress!!! (0 - 360)
     dec:=sin(DegToRad(Position))*(23.5+AngleFromEcliptic);
     Calc_XY_From_RA_Dec(Position/15+Ugol_Rot/15,Dec,x,y);
     (Sender as TBitmap).Canvas.Draw(x-BMP.Width div 2,y-BMP.Height div 2,BMP);
end;

procedure TSky.Draw_Grid(Sender:TObject;StepLong,StepLat:word);
var
   a,d : extended;
   x,y : integer;

begin
     d:=-90;     //
     repeat
           a:=0; // 
           repeat
                 Calc_XY_From_RA_Dec(a/15+Ugol_Rot/15,d,x,y);
                 (Sender as TBitmap).Canvas.Pixels[x,y]:=clBlue;
                 a:=a+1;
           until a>360;
           d:=d+StepLong;
     until d>=90;

     a:=0;
     repeat
           d:=-90;
           repeat
                 Calc_XY_From_RA_Dec(a/15+Ugol_Rot/15,d,x,y);
                 (Sender as TBitmap).Canvas.Pixels[x,y]:=clBlue;
                 d:=d+1;
           until d>=90;
           a:=a+StepLat;
     until a>=360;
end;

procedure TSky.Draw_Sun(Sender:TObject;BMP:TBitmap);
var
   st:TSYSTEMTIME;
begin
     GetSystemTime(ST);
     TraceCoordEcliptic(Sender,BMP,Sun_Position(ST.wYear,ST.wMonth,ST.wDay),0);
end;

procedure TSky.Draw_Moon(Sender:TObject;BMP:TBitmap);
var
   st:TSYSTEMTIME;
   MA:Extended;
begin
     GetSystemTime(ST);
     MA:=MoonAge(ST.wYear,ST.wMonth,ST.wDay);
     BMP.Canvas.Brush.Style:=bsClear;
     BMP.Canvas.Font.Color:=clRed;
     BMP.Canvas.Font.Size:=10;
     BMP.Canvas.TextOut(8,8,IntToStr(round(MA)));

     TraceCoordEcliptic(Sender,BMP,Moon_Position(ST.wYear,ST.wMonth,ST.wDay),0);
end;


end.

